// BWCurveCanvas.m -- an interactive View that shows a bezier path and movable
//		      control points

// Copyright (C) 2002 Borkware
// We use the BSD License.  Check out http://borkware.com/license

#import "BWCurveCanvas.h"

// how big the control points are
#define BLOB_SIZE 5.0


// magic flag for draggedPoint if we're not currently dragging
#define NOT_DRAGGING -1


@implementation BWCurveCanvas

// lay out the default point positions.  these values generated by
// the hyper-precise "Mess around with numbers until it looks good"
// algorithm.

- (NSPoint *) makeDefaultPoints
{
    NSPoint *pts;
    pts = malloc (sizeof(NSPoint) * 7);
    
    // start, control1, control2, end, control1, control2, end
    pts[0].x = 5;
    pts[0].y = 5;
    
    pts[1].x = 50;
    pts[1].y = 100;
    
    pts[2].x = 90;
    pts[2].y = 100;
    
    pts[3].x = 130;
    pts[3].y = 50;
    
    pts[4].x = 170;
    pts[4].y = 5;
    
    pts[5].x = 270;
    pts[5].y = 20;
    
    pts[6].x = 300;
    pts[6].y = 80;
    
    return (pts);

} // makeDefaultPoints



- (id) initWithFrame: (NSRect) frame
{
    if (self = [super initWithFrame: frame]) {
	draggedPoint = NOT_DRAGGING;
	flatness = 1.0;
	fill = -1;
	points = [self makeDefaultPoints];
    }

    return (self);

} // initWithFrame



// draw a black circle with a red border (to make it a little easier to see)

- (void) drawPoint: (NSPoint) pt
{
    NSBezierPath *path = [[NSBezierPath alloc] init];

    [path appendBezierPathWithArcWithCenter: pt 
	  radius: BLOB_SIZE / 2.0
	  startAngle: 0.0  
	  endAngle: 360.0];
    
    [[NSColor blackColor] set];
    [path fill];
    [[NSColor redColor] set];
    [path stroke];

} // drawPoint


- (void) drawRect: (NSRect) rect
{
    int i;
    NSBezierPath *path = [[NSBezierPath alloc] init];

    [[NSColor whiteColor] set];
    [NSBezierPath fillRect: rect];
    [[NSColor blackColor] set];
    [NSBezierPath strokeRect: rect];

    [path moveToPoint: points[0]];
    [path curveToPoint: points[3]  controlPoint1: points[1]  controlPoint2: points[2]];
    
    [path moveToPoint: points[3]];
    [path curveToPoint: points[6] controlPoint1: points[4]  controlPoint2: points[5]];
    
    [path setLineWidth: 2];
    [path setFlatness: flatness];

    if (fill != -1) {
	[[NSColor redColor] set];
	[path setWindingRule: fill];
	[path fill];
	[[NSColor blackColor] set];
    }
    
    [path stroke];
    [path release];

    path = [[NSBezierPath alloc] init];

    [path moveToPoint: points[0]];

    for (i = 1; i < 7; i++) {
	[path lineToPoint: points[i]];
    }
	
    [[NSColor grayColor] set];
    [path setLineWidth: 0.5];

    [path stroke];

    [path release];

    for (i = 0; i < 7; i++) {
	[self drawPoint: points[i]];
    }

} // drawRect



// mouse down handler.  initiate a drag operation if it's in a control point

- (void) mouseDown: (NSEvent *) event
{
    int i;

    draggedPoint = NOT_DRAGGING;

    for (i = 0; i < 7; i++) {
	NSPoint center = points[i];
	NSPoint click = [self convertPoint: [event locationInWindow] 
			      fromView: nil];

	NSRect rect = NSMakeRect (center.x - BLOB_SIZE / 2,
				  center.y - BLOB_SIZE / 2,
				  BLOB_SIZE, BLOB_SIZE);

	if (NSPointInRect(click, rect)) {
	    draggedPoint = i;
	    break;
	}
    }

} // mouseDown



// if we're in a drag operation, move the point around and redraw.
// Gives the illusion of live updating

- (void) mouseDragged: (NSEvent *) event
{
    if (draggedPoint != NOT_DRAGGING) {
	NSPoint click = [self convertPoint: [event locationInWindow] 
			      fromView: nil];
	points[draggedPoint] = click;
	[self setNeedsDisplay: YES];
    }

} // mouseDragged



// yay!  We're done dragging

- (void) mouseUp: (NSEvent *) event
{
    if (draggedPoint != NOT_DRAGGING) {
	NSPoint click = [self convertPoint: [event locationInWindow] 
			      fromView: nil];
	points[draggedPoint] = click;
	[self setNeedsDisplay: YES];

	if (!NSPointInRect(click, [self bounds])) {
	    [delegate pointOutOfFrame];
	}
    }

    draggedPoint = NOT_DRAGGING;

} // mouseUp



// if there are any points that are outside of the canvas, bring 'em in.

- (void) pinPoints
{
    int i;
    NSRect bounds = [self bounds];

    // bring the pinned points so that the larger blob indicating the
    // point are completely visible
    bounds = NSInsetRect (bounds, BLOB_SIZE, BLOB_SIZE);

    for (i = 0; i < 7; i++) {
	if (!NSPointInRect(points[i], bounds)) {
	    points[i].x = (points[i].x < bounds.origin.x) ? bounds.origin.x 
						          : points[i].x;
	    points[i].y = (points[i].y < bounds.origin.y) ? bounds.origin.y
							  : points[i].y;

	    points[i].x = (points[i].x > bounds.origin.x
					 + bounds.size.width) 
		? bounds.origin.x + bounds.size.width
		: points[i].x;

	    points[i].y = (points[i].y > bounds.origin.y
					 + bounds.size.height) 
		? bounds.origin.y + bounds.size.height
		: points[i].y;

	    [self setNeedsDisplay: YES];
	}
    }

    // tell folks we've shifted points.  Maybe the UI will disable a
    // button or something

    [delegate allPointsInFrame];

} // pinPoints



// see if any points lie outside of the bounds

- (BOOL) anyPointsOutsideRect: (NSRect) rect
{
    int i;
    BOOL result = NO;

    for (i = 0; i < 7; i++) {
	if (!NSPointInRect (points[i], rect)) {
	    result = YES;
	    break;
	}
    }

    return (result);

} // anyPointsOutsideRect



// we're resizing.  See if any points now lie outside of the bounds.
// if so, tell the delegate
- (void) resizeWithOldSuperviewSize: (NSSize) oldFrameSize
{
    [super resizeWithOldSuperviewSize: oldFrameSize];

    if ([self anyPointsOutsideRect: [self bounds]]) {
	[delegate pointOutOfFrame];
    } else {
	[delegate allPointsInFrame];
    }

} // resizeWithOldSuperviewSize



// clean up our mess

- (void) dealloc
{
    free (points);
    [super dealloc];
} // dealloc



// set the curve flatness (error tolerance when stroking or filling

- (void) setFlatness: (float) f
{
    flatness = f;
    [self setNeedsDisplay: YES];

} // setFlatness



// set the winding rule - how it fills overlapping reas

- (void) setFill: (int) windingRule
{
    fill = windingRule;
    [self setNeedsDisplay: YES];
} // setFill



// restore the points after the user has mangled them

- (void) resetPoints
{
    free (points);
    points = [self makeDefaultPoints];

} // resetPoints


@end // BWCurveCanvas


