// BWImageCanvas.m -- draw an image of a block of cheese, and let the user
//		      scale, rotate, and translate the image

// Copyright (C) 2002 Borkware
// We use the BSD License.  Check out http://borkware.com/license

#import "BWImageCanvas.h"


@implementation BWImageCanvas


// set up defaults and other goodies

- (id) initWithFrame: (NSRect) rect
{
    if (self = [super initWithFrame: rect]) {
	scaleX = 1.0;
	scaleY = 1.0;
	translateX = 1.0;
	translateY = 1.0;
	rotation = 0.0;

	NSBundle *me = [NSBundle mainBundle];
	NSString *path = [me pathForResource: @"dazed-cheese"
			     ofType: @"jpg"];
	image = [[NSImage alloc] initWithContentsOfFile: path];
    }
    
    return (self);

} // initWithFrame



// Draw blob.  This is drawn at the origin point of the image.
// this will get mutated by the transform

- (void) drawPoint: (NSPoint) pt  isControl: (BOOL) control
{
    NSBezierPath *path = [[NSBezierPath alloc] init];

#define BLOB_SIZE 7
    [path appendBezierPathWithArcWithCenter: pt 
	  radius: BLOB_SIZE / 2.0
	  startAngle: 0.0 
	  endAngle: 360.0];
    
    [[NSColor blackColor] set];
    [path fill];
    [[NSColor redColor] set];
    [path stroke];

} // drawPoint



// get the current transform after all the operations have been
// applied.  The origin of the image is in the middle of the view
// to make rotations more understandable

- (NSAffineTransform *) currentTransform
{
    NSAffineTransform *transform;
    NSRect bounds = [self bounds];

    // to put the "zero" point in the middle of the picture, we'll
    // set the origin to be minus half the width and height

    transform = [NSAffineTransform transform];

    // center it
    [transform translateXBy: bounds.size.width / 2.0  yBy: bounds.size.height / 2.0];
    [transform scaleXBy: scaleX  yBy: scaleY];
    [transform rotateByDegrees: rotation];
    
    // apply translation
    [transform translateXBy: translateX  yBy: -translateY];

    return (transform);

} // currentTransform



- (void) drawRect: (NSRect) rect
{
    NSRect imageBounds;
    
    imageBounds.origin = NSZeroPoint;
    imageBounds.size = [image size];

    NSAffineTransform *transform;

    // be sloppy and redraw the whole shebang
    rect = [self bounds];

    // erase the background to white
    [[NSColor whiteColor] set];
    [NSBezierPath fillRect: rect];

    // we're concatting the transform to the current graphics state

    [NSGraphicsContext saveGraphicsState]; {

	transform = [self currentTransform];
	[transform concat];

	[image drawAtPoint: NSZeroPoint
	       fromRect: imageBounds
		   operation: NSCompositeCopy
		   fraction: 1.0];

	// draw point at image's zero,zero point
	[self drawPoint: NSZeroPoint  isControl: NO];

	// draw the axes
	[[NSColor grayColor] set];
	[NSBezierPath strokeLineFromPoint: NSMakePoint (-5000, 0)
		      toPoint: NSMakePoint (5000, 0)];
	[NSBezierPath strokeLineFromPoint: NSMakePoint (0, -5000)
		      toPoint: NSMakePoint (0, 5000)];

    } [NSGraphicsContext restoreGraphicsState];

    [[NSColor blackColor] set];

    // draw a spot in the middle of the view, which is the point
    // of rotation
    NSRectFill (NSMakeRect(rect.origin.x + rect.size.width / 2.0,
			   rect.origin.y + rect.size.height / 2.0, 2.0, 2.0));

    // put a nice plain frame around the view
    [NSBezierPath strokeRect: rect];
    
} // drawRect



- (void) setImage: (NSImage *) newImage
{
    [newImage retain];
    [image release];
    image = newImage;
} // setImage



- (void) setScaleX: (float) value
{
    scaleX = value;
    [self setNeedsDisplay: YES];
} // setScaleX



- (float) scaleX
{
    return (scaleX);
} // scaleX



- (void) setScaleY: (float) value
{
    scaleY = value;
    [self setNeedsDisplay: YES];
} // setScaleY



- (float) scaleY
{
    return (scaleY);
} // scaleY



- (void) setTranslateX: (float) value
{
    translateX = value;
    [self setNeedsDisplay: YES];
} // setTranslateX



- (float) translateX
{
    return (translateX);
} // translateX



- (void) setTranslateY: (float) value
{
    translateY = value;
    [self setNeedsDisplay: YES];
} // setTranslateY



- (float) translateY
{
    return (translateY);
} // translateY



- (void) setRotation: (float) value
{
    rotation = value;
    [self setNeedsDisplay: YES];
} // setRotation



- (float) rotation
{
    return (rotation);
} // rotation



@end // BWImageCanvas


